import sys
import os
import re
import common
forge = sys.modules["forge"]

def setup(module):
	manifest = 'ext/fbx/manifest.cc'
	with open(manifest, 'w') as outfile:
		outfile.write("")

	for fbx_source in reversed(sorted(forge.fbx_sources)):
		fbx_source_version = fbx_source[0]
#		fbx_source_suffix = fbx_source[1]
		fbx_source_inc = fbx_source[2]
		fbx_source_libs = fbx_source[3]

		if not os.path.exists(fbx_source_inc) or not os.path.exists(fbx_source_libs):
			forge.color_on(0, forge.BLUE)
			sys.stdout.write(" -" + fbx_source_version)
			forge.color_off()
			continue

		if fbx_source_version == "":
			sys.stdout.write(" /")
		else:
			sys.stdout.write(" ")
			sys.stdout.write(fbx_source_version)

		srcList = [	"SurfaceAccessibleFbx",
					"fbx.pmh",
					"fbxDL" ]

		variant = fbx_source_version
		if variant != "":
			variantPath = module.modPath + '/' + variant
			if os.path.lexists(variantPath) == 0:
				os.symlink('.', variantPath)

			srcListVariant = []
			for src in srcList:
				srcListVariant += [ variant + '/' + src ]
			srcList = srcListVariant

		fbx_version = variant
		fbx_version_path = fbx_source_inc + '/FBX/FbxCoreAbstract/Foundation.h'
		if os.path.exists(fbx_version_path):
			for line in open(fbx_version_path):
				if 'ALEMBIC_LIBRARY_VERSION ' in line:
					fbx_version = line.split(' ')[2].rstrip()
					break

		if fbx_version != variant:
			sys.stdout.write("=" + fbx_version)

		dllname = "fexFbxDL" + variant

		with open(manifest, 'a') as outfile:
			suffix = ""
			if variant != "":
				suffix = "." + variant
			outfile.write('\tspManifest->catalog<String>(\n'+
					'\t\t\t"SurfaceAccessibleI.SurfaceAccessibleFbx.fe.fbx'+
					suffix+'")=\n'+
					'\t\t\t"fexFbxDL'+variant+'";\n');

		dll = module.DLL( dllname, srcList )

		for src in srcList:
			relativePath = forge.RelativePath(forge.rootPath,module.modPath)
			modObjPath = os.path.join(forge.objPath, relativePath)
			objName = modObjPath + '/' + src
			if re.compile(r'(.*)\.pmh$').match(src):
				objName += '.gch'
			else:
				objName += forge.objsuf
			srcTarget = forge.FindTarget(objName)
			srcTarget.includemap = { 'fbx' : fbx_source_inc }

		dll.linkmap = { "fbx" : "" }

		dll.linkmap["fbx"] += " -Xlinker -rpath '" + fbx_source_libs + "/'"
		dll.linkmap["fbx"] += " -L" + fbx_source_libs + ' -lfbxsdk'

		deplibs = forge.corelibs + [
					"fexSurfaceDLLib" ]

		forge.deps( [ dllname + "Lib" ], deplibs )

	forge.tests += [
		("inspect.exe",		"fexFBXDL",				None,		None) ]

	module.Module('test')
