/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __opengl_DrawOpenGL_h__
#define __opengl_DrawOpenGL_h__

#define DGL_CYLINDER_GLU	FALSE	// replace default with glu version
#define DGL_SPHERE_GLU		TRUE	// replace default with glu version

namespace fe
{

/**************************************************************************//**
	@brief OpenGL-specific implementations for DrawI

	@ingroup opengl
*//***************************************************************************/
class FE_DL_EXPORT DrawOpenGL: public DrawCommon,
		public Initialize<DrawOpenGL>
{
	public:
					DrawOpenGL();
virtual				~DrawOpenGL();

		void		initialize(void);

virtual	BWORD		isDirect(void) const				{ return m_isDirect; }
virtual	void		flush(void);

virtual void		pushMatrixState(void);
virtual void		popMatrixState(void);

virtual void		setBrightness(Real a_brightness);
virtual void		setContrast(Real a_contrast);

virtual void		setDrawMode(sp<DrawMode> spMode);

virtual	void		setFontBase(U32 index)			{ m_fontbase=index; };

virtual void		drawPoints(const SpatialVector *vertex,
							const SpatialVector *normal,U32 vertices,
							BWORD multicolor,const Color *color,
							sp<DrawBufferI> spDrawBuffer);

virtual void		drawLines(const SpatialVector *vertex,
							const SpatialVector *normal,U32 vertices,
							StripMode strip,BWORD multicolor,
							const Color *color,
							const Vector3i *element,U32 elementCount,
							sp<DrawBufferI> spDrawBuffer);

virtual	void		drawTriangles(const SpatialVector *vertex,
							const SpatialVector *normal,
							const Vector2f *texture,U32 vertices,
							StripMode strip,BWORD multicolor,
							const Color *color,
							sp<DrawBufferI> spDrawBuffer);

virtual void		drawRectangles(const SpatialVector *vertex,U32 vertices,
						BWORD multicolor,const Color *color);

virtual	void		drawAlignedText(const SpatialVector &location,
							const String text,const Color &color);

#if DGL_CYLINDER_GLU
virtual	void		drawCylinder(const SpatialTransform &transform,
							const SpatialVector *scale,
							Real baseScale,const Color &color,U32 slices)
					{	drawCylinderGLU(transform,scale,baseScale,
								color,slices); }
#endif

#if DGL_SPHERE_GLU
virtual	void		drawSphere(const SpatialTransform &transform,
							const SpatialVector *scale,const Color &color)
					{	drawSphereGLU(transform,scale,color); }
#endif

virtual void		drawTransformedTriangles(
							const SpatialTransform &transform,
							const SpatialVector *scale,
							const SpatialVector *vertex,
							const SpatialVector *normal,
							const Vector2f *texture,
							U32 vertices,StripMode strip,BWORD multicolor,
							const Color *color,
							sp<DrawBufferI> spDrawBuffer);

virtual	void		drawRaster(sp<ImageI> spImageI,
						const SpatialVector& location);

virtual	sp<DrawBufferI>	createBuffer(void);

	protected:

virtual	void		createDefaultView(void);

	private:

	class Buffer: public DrawBufferI
	{
		public:
						Buffer(void);
	virtual				~Buffer(void);
			void		releaseAll(void);

			sp<DrawI>	m_spDrawI;
			GLuint		m_vboVertex;
			GLuint		m_vboNormal;
			GLuint		m_vboTexture;
			GLuint		m_vboColor;
	};

	class Font: public Component
	{
		public:
							Font(void);
	virtual					~Font(void);

			I32				m_fontBase;
			I32				m_fontRange;
			XFontStruct*	m_pFontStruct;
	};

		void		drawLinesInternal(const SpatialVector *vertex,
							const SpatialVector *normal,
							const Vector2f *texture,U32 vertices,
							StripMode strip,BWORD multicolor,
							const Color *color,
							const Vector3i *element,U32 elementCount,
							sp<DrawBufferI> spDrawBuffer);

		void		assureFont(void);

		void		threadLock(void);
		void		threadUnlock(void);

		void		bindTexture(sp<ImageI> spImageI);

		void		drawCylinderGLU(const SpatialTransform &transform,
							const SpatialVector *scale,
							Real baseScale,const Color &color,U32 slices);

		void		drawSphereGLU(const SpatialTransform &transform,
							const SpatialVector *scale,const Color &color);

		void		beginArrays(const SpatialVector *vertex,
							const SpatialVector *normal,
							const Vector2f *texture,U32 vertices,
							BWORD multicolor,const Color *color,
							sp<DrawBufferI> spDrawBuffer,
							BWORD normals,BWORD blending);
		void		endArrays(const Vector2f *texture,
							BWORD multicolor,const Color *color,
							sp<DrawBufferI> spDrawBuffer,
							BWORD normals);
		void		drawPolys(U32 grain,const SpatialVector *vertex,
							const SpatialVector *normal,
							const Vector2f *texture,
							U32 vertices,StripMode strip,
							BWORD multicolor,const Color *color,
							sp<DrawBufferI> spDrawBuffer);

		void		setupLights(void);
		void		createDefaultTexture(void);
		void		useDefaultTexture(void);
		void		pushAntialiasPoints(void);
		void		pushAntialiasLines(void);
		void		pushBlending(void);
		void		activateBlending(void);
		void		checkForError(String a_location);
		void		queryFormat(sp<ImageI> spImageI,GLenum& format,
							GLenum& type,GLint& components);

		void		setup(DrawMode::DrawStyle style,BWORD lit,BWORD dual,
							BWORD texture,BWORD frontcull,BWORD backcull,
							BWORD zBuffer);

		sp<ImageI>			m_currentTexture;
		U32					m_fontbase;
		BWORD				m_isDirect;
		BWORD				m_lockThreads;
		GLUquadric*			m_pQuadric;
		GLubyte				m_texture[64][64][3];

static	FE_DL_PUBLIC	sp<Font>	ms_spFont;
						sp<Font>	m_spFontHold;
};

} // namespace

#endif /* __opengl_DrawOpenGL_h__ */
