/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

#define FE_SACP_DEBUG	FALSE

namespace fe
{

void SurfaceAttrCopyOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	//* Point
	catalog<bool>("PointPosition")=false;
	catalog<String>("PointPosition","label")="Position";
	catalog<String>("PointPosition","page")="Point";

	catalog<bool>("PointNormal")=false;
	catalog<String>("PointNormal","label")="Normal";
	catalog<String>("PointNormal","page")="Point";

	catalog<String>("PointAttributes")="";
	catalog<String>("PointAttributes","label")="Attributes";
	catalog<String>("PointAttributes","page")="Point";
	catalog<String>("PointAttributes","hint")=
		"Space delimited list of attributes to copy."
		"  Any token containing an equals sign indicates"
		" a name conversion of the form 'to=from'.";

	//* Primitive
	catalog<bool>("PrimitiveNormal")=false;
	catalog<String>("PrimitiveNormal","label")="Normal";
	catalog<String>("PrimitiveNormal","page")="Primitive";

	catalog<String>("PrimitiveAttributes")="";
	catalog<String>("PrimitiveAttributes","label")="Attributes";
	catalog<String>("PrimitiveAttributes","page")="Primitive";
	catalog<String>("PrimitiveAttributes","hint")=
			catalog<String>("PointAttributes","hint");

	catalog< sp<Component> >("Input Surface");

	catalog< sp<Component> >("Source Surface");
}

void SurfaceAttrCopyOp::handle(Record& a_rSignal)
{
#if FE_SACP_DEBUG
	feLog("SurfaceAttrCopyOp::handle\n");
#endif

	const BWORD pointPosition=catalog<bool>("PointPosition");
	const BWORD pointNormal=catalog<bool>("PointNormal");
	const String pointAttributes=catalog<String>("PointAttributes");

	const BWORD primitiveNormal=catalog<bool>("PrimitiveNormal");
	const String primitiveAttributes=catalog<String>("PrimitiveAttributes");

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessibleI> spSourceAccessible;
	if(!access(spSourceAccessible,"Source Surface")) return;

	const I32 classCount=2;
	for(I32 classIndex=0;classIndex<classCount;classIndex++)
	{
		const Element element=classIndex? e_primitive: e_point;

		BWORD needPosition=(!classIndex && pointPosition);
		BWORD needNormal=classIndex? primitiveNormal: pointNormal;
		String attributes=classIndex? primitiveAttributes: pointAttributes;

		while(TRUE)
		{
			sp<SurfaceAccessorI> spOutputAccessor;
			sp<SurfaceAccessorI> spSourceAccessor;

			if(needPosition)
			{
				needPosition=FALSE;

				if(!access(spOutputAccessor,spOutputAccessible,
						element,e_position)) continue;
				if(!access(spSourceAccessor,spSourceAccessible,
						element,e_position)) continue;

#if FE_SACP_DEBUG
				feLog("copy position\n");
#endif
			}
			else if(needNormal)
			{
				needNormal=FALSE;

				if(!access(spOutputAccessor,spOutputAccessible,
						element,e_normal,e_warning)) continue;
				if(!access(spSourceAccessor,spSourceAccessible,
						element,e_normal,e_warning)) continue;

#if FE_SACP_DEBUG
				feLog("copy normal\n");
#endif
			}
			else
			{
				String token=attributes.parse();
				if(token.empty())
				{
					break;
				}

				const String toAttr=token.parse("","=");
				const String fromAttr=token.empty()? toAttr: token.prechop("=");

				if(!access(spOutputAccessor,spOutputAccessible,
						element,toAttr,e_warning)) continue;
				if(!access(spSourceAccessor,spSourceAccessible,
						element,fromAttr,e_warning)) continue;

#if FE_SACP_DEBUG
				feLog("copy \"%s\" to \"%s\"\n",
						fromAttr.c_str(),toAttr.c_str());
#endif
			}

			I32 elementCount=spOutputAccessor->count();
			const I32 sourceCount=spSourceAccessor->count();
			if(elementCount>sourceCount)
			{
				elementCount=sourceCount;
			}

			const String attrType=spSourceAccessor->type();

			if(attrType=="String")
			{
				for(I32 elementIndex=0;elementIndex<elementCount;elementIndex++)
				{
					if(interrupted())
					{
						classIndex=classCount;
						break;
					}

					spOutputAccessor->set(elementIndex,
							spSourceAccessor->string(elementIndex));
				}
			}
			else if(attrType=="Integer")
			{
				for(I32 elementIndex=0;elementIndex<elementCount;elementIndex++)
				{
					if(interrupted())
					{
						classIndex=classCount;
						break;
					}

					spOutputAccessor->set(elementIndex,
							spSourceAccessor->integer(elementIndex));
				}
			}
			else if(attrType=="Real")
			{
				for(I32 elementIndex=0;elementIndex<elementCount;elementIndex++)
				{
					if(interrupted())
					{
						classIndex=classCount;
						break;
					}

					spOutputAccessor->set(elementIndex,
							spSourceAccessor->real(elementIndex));
				}
			}
			else if(attrType=="Vector3")
			{
				for(I32 elementIndex=0;elementIndex<elementCount;elementIndex++)
				{
					if(interrupted())
					{
						classIndex=classCount;
						break;
					}

					spOutputAccessor->set(elementIndex,
							spSourceAccessor->spatialVector(elementIndex));

//					feLog("SurfaceAttrCopyOp::handle %d/%d %s\n",
//							elementIndex,elementCount,
//							c_print(spSourceAccessor->spatialVector(
//							elementIndex)));
				}
			}
		}
	}

#if FE_SACP_DEBUG
	feLog("SurfaceAttrCopyOp::handle done\n");
#endif
}

} // namespace
