/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

#define FE_TBO_DEBUG		FALSE

using namespace fe;

void TubeOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog< sp<Component> >("Input Curves");

	//* NOTE no auto-copy
	catalog< sp<Component> >("Output Surface");
	catalog<String>("Output Surface","IO")="output";
}

void TubeOp::handle(Record& a_rSignal)
{
#if FE_TBO_DEBUG
	feLog("TubeOp::handle node \"%s\"\n",name().c_str());
#endif

	String& rSummary=catalog<String>("summary");
	rSummary="";

	sp<DrawI> spDrawI;
	if(!accessDraw(spDrawI,a_rSignal)) return;

	sp<SurfaceAccessibleI> spInputAccessible;
	if(!access(spInputAccessible,"Input Curves")) return;

	sp<SurfaceAccessorI> spInputVertices;
	if(!access(spInputVertices,spInputAccessible,
				e_primitive,e_vertices)) return;

	sp<SurfaceAccessorI> spInputColor;
	access(spInputColor,spInputAccessible,e_point,e_color,e_warning);

	sp<SurfaceAccessorI> spInputRadius;
	access(spInputRadius,spInputAccessible,e_point,"radius",e_warning);

	const I32 primitiveCount=spInputVertices->count();

#if FE_TBO_DEBUG
	feLog("TubeOp::handle primitiveCount %d\n",primitiveCount);
#endif

	for(I32 primitiveIndex=0;primitiveIndex<primitiveCount;primitiveIndex++)
	{
		const I32 subCount=spInputVertices->subCount(primitiveIndex);

#if FE_TBO_DEBUG
		feLog("TubeOp::handle primitive %d/%d subCount %d\n",
				primitiveIndex,primitiveCount,subCount);
#endif

		SpatialVector line[subCount];
		Color color[subCount];
		Real radius[subCount];

		for(I32 subIndex=0;subIndex<subCount;subIndex++)
		{
			line[subIndex]=
					spInputVertices->spatialVector(primitiveIndex,subIndex);

			const I32 pointIndex=
					spInputVertices->integer(primitiveIndex,subIndex);

			if(spInputRadius.isValid())
			{
				radius[subIndex]=
						spInputRadius->real(pointIndex);
			}

			if(spInputColor.isValid())
			{
				color[subIndex]=
						spInputColor->spatialVector(pointIndex);
			}
		}

		//* TODO
		const SpatialVector towardCamera(0.0,0.0,0.0);

		OperateCommon::drawTube(spDrawI,line,NULL,
				spInputRadius.isValid()? radius: NULL,
				subCount,OperateCommon::e_tube,TRUE,
				spInputColor.isValid()? color: NULL,
				towardCamera);
	}

	rSummary.sPrintf("%d",primitiveCount);

#if FE_TBO_DEBUG
	feLog("TubeOp::handle node \"%s\" done\n",name().c_str());
#endif
}
